#!/bin/sh -efu
#
# Shell implementation of man.
#
# Copyright (c) 2009  Alexey Gladkov <gladkov.alexey@gmail.com>
#

. shell-args
. shell-getopt
. man-sh-functions

use_sect=
format=
allmatch=
location=

show_help() {
	cat <<EOF
Usage: $PROG [options] [section] name...
  -a              find all matching entries;
  -c              do not use cat file;
  -t              use troff to format pages for printing;
  -w, --path      print location of man page(s) that would be displayed;
  -W              as for -w, but display filenames only;
  -C FILE         Specify the configuration file to use;
  -M PATH         set search path for manual pages to 'PATH';
  -P PAGER        use program 'PAGER' to display pages;
  -S LIST         colon separated section list;

  -V, --version   print program version and exit;
  -h, --help      show this text and exit.

EOF
	exit
}

print_version() {
	cat <<EOF
$PROG version $PROG_VERSION
Written by Alexey Gladkov <gladkov.alexey@gmail.com>

Copyright (C) 2009  Alexey Gladkov <gladkov.alexey@gmail.com>
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
EOF
	exit
}

TEMP=`getopt -n $PROG -o 'a,c,C:,M:,t,w,W,h,V' -l 'path,help,version' -- "$@"` ||
	show_usage
eval set -- "$TEMP"
while :; do
	case "$1" in
		-a) allmatch=1
			;;
		-C)
			shift
			config_file="$1"
			;;
		-M)
			shift
			MANPATH="$1"
			ignore="$ignore manpath "
			;;
		-P)
			shift
			PAGER="$1"
			ignore="$ignore pager "
			;;
		-S)
			shift
			MANSECT="$1"
			ignore="$ignore mansect "
			;;
		-t) format='ps'
			;;
		-w|--path) location='path'
			;;
		-W) location='name'
			;;
		-h|--help) show_help
			;;
		-V|--version) print_version
			;;
		-c) # ignore option
			;;
		--) shift; break
			;;
		*) fatal "Unrecognized option: $1"
			;;
	esac
	shift
done

read_conf
set_lang
set_pagesize

found=0
manfiles=
manfiles_info=
candidate() {
	[ -f "$1" ] || return 0
	local f
	for f in $manfiles; do
		[ "$1" != "$f" ] || return 0
	done
	manfiles="${manfiles:+$manfiles
}$1"
	manfiles_info="${manfiles_info:+$manfiles_info }$name($sect)"
	found=$(($found+1))

	[ -n "$allmatch" ] ||
		return 1
}

find_manfiles() {
	local path sect suff name="$1"
	for path in $MANPATH; do
		for sect in $MANSECT; do
			if [ -n "$use_sect" ]; then
				[ "$sect" = "$use_sect" ] ||
					continue
			fi
			for suff in '' $COMPRES; do
				candidate "$path/man$sect/$name.${sect}${suff}" || break 3
			done
		done
	done
}

if [ "$#" -eq 0 ]; then
	[ "$location" = 'path' ] &&
		printf '%s\n' "$MANPATH" |tr ' ' ':' ||
		printf 'What manual page do you want?\n'
	exit
fi

if [ "$#" -gt 1 ]; then
	for sect in $MANSECT; do
		if [ "$1" = "$sect" ]; then
			use_sect="$1"
			shift
			break
		fi
	done
fi

export MAN_CONFIG="$config_file"

for fn; do
        found=0
	manfiles=
	manfiles_info=

	if [ -f "$fn" ]; then
		manfiles="$fn"
		found=1
	else
		find_manfiles "$fn"
	fi

	if [ "$found" -eq 0 ]; then
		printf 'No manual entry for %s\n' "$fn"
		continue
	fi

	if [ -n "$location" ]; then
		for manfile in $manfiles; do
			case "$location" in
				path) printf '%s\n' "$manfile" ;;
				name) printf '%s\n' "${manfile##*/}" ;;
			esac
			[ -n "$allmatch" ] ||
				break
		done
		continue
	fi

	for manfile in $manfiles; do
		if [ "$format" != 'ps' ]; then
			man-source.sh -l "$ll" -p "$pl" "$manfile" |
				$NROFF |
				$PAGER
		else
			man-source.sh -l "$ll" -p "$pl" "$manfile" |
				$TROFF
		fi

		[ -n "$allmatch" ] ||
			break
	done
done
exit
tput
